// ======================================= //
//                                         //
//                    //
//            SendInput()          //
//            2025-04-05             //
//                                         //
//               HiAsm               //
//             www.hiasm.com               //
//                                         //
//            : Netspirit             //
//        E-mail: netspirit@meta.ua        //
//                                         //
// ======================================= //

unit SendInp;

interface

uses
  Windows, KOL;


const
  INPUT_MOUSE = 0; //   Windows  Delphi 4   INPUT_MOUSE, INPUT_KEYBOARD
  INPUT_KEYBOARD = 1;
  INPUT_HARDWARE = 2;

  KEYEVENTF_EXTENDEDKEY = $0001;
  KEYEVENTF_KEYUP = $0002;
  KEYEVENTF_UNICODE = $0004;
  KEYEVENTF_SCANCODE = $0008;

  MAPVK_VK_TO_VSC = 0;
  MAPVK_VSC_TO_VK = 1;
  MAPVK_VK_TO_CHAR = 2;
  MAPVK_VSC_TO_VK_EX = 3;

  EXT_KEYS: array [0..9] of Byte =
  (
    VK_INSERT, VK_DELETE, VK_HOME,
    VK_END, VK_PRIOR, VK_NEXT,
    VK_LEFT, VK_UP, VK_DOWN, VK_RIGHT
  );

type

  PMouseInput = ^TMouseInput;
  tagMOUSEINPUT = record
    dx: Longint;
    dy: Longint;
    mouseData: DWord;
    dwFlags: DWord;
    time: DWord;
    dwExtraInfo: DWord;
  end;
  TMouseInput = tagMOUSEINPUT;

  PKeybdInput = ^TKeybdInput;
  tagKEYBDINPUT = record
    wVk: Word;
    wScan: Word;
    dwFlags: DWord;
    time: DWord;
    dwExtraInfo: DWord;
  end;
  TKeybdInput = tagKEYBDINPUT;

  PHardwareInput = ^THardwareInput;
  tagHARDWAREINPUT = record
    uMsg: DWord;
    wParamL: Word;
    wParamH: Word;
  end;
  THardwareInput = tagHARDWAREINPUT;

  PInput = ^TInput;
  tagINPUT = record
    Itype: DWord;
    case Integer of
      0: (mi: TMouseInput);
      1: (ki: TKeybdInput);
      2: (hi: THardwareInput);
  end;
  TInput = tagINPUT;


  function IsExtendedKey(VKCode: Byte): Boolean;
  procedure EmulateKeyDown(VKCode: Byte);
  procedure EmulateKeyUp(VKCode: Byte);
  procedure EmulateKeyPress(VKCode: Byte);
  procedure EmulateKeyDownSC(ScanCode: Word);
  procedure EmulateKeyUpSC(ScanCode: Word);
  procedure EmulateKeyPressSC(ScanCode: Word);
  procedure EmulateKeyPressW(C: WideChar);

  procedure EmulateString(S: AnsiString); overload;
  procedure EmulateStringW(S: WideString); overload;


  function SendInput(cInputs: UINT; var pInputs: TInput; cbSize: Integer): UINT; stdcall; external 'user32.dll' name 'SendInput';

implementation


function IsExtendedKey(VKCode: Byte): Boolean;
var
  I: Integer;
begin
  for I := Low(EXT_KEYS) to High(EXT_KEYS) do
  begin
    if EXT_KEYS[I] = VKCode then
    begin
      Result := True;
      Exit;
    end;
  end;
  Result := False;
end;

procedure EmulateKeyDown(VKCode: Byte);
var
  Input: TInput;
begin
  FillChar(Input, SizeOf(TInput), 0);
  with Input do
  begin
    Itype := INPUT_KEYBOARD;
    ki.wVk := VKCode;
    //  :
    //      EXT_KEYS,
    //       Shift, Ctrl, Alt
    //    ,   
    if IsExtendedKey(VKCode) then
      ki.dwFlags := KEYEVENTF_EXTENDEDKEY;
  end;
  SendInput(1, Input, SizeOf(TInput));
end;

procedure EmulateKeyUp(VKCode: Byte);
var
  Input: TInput;
begin
  FillChar(Input, SizeOf(TInput), 0);
  with Input do
  begin
    Itype := INPUT_KEYBOARD;
    ki.wVk := VKCode;
    ki.dwFlags := KEYEVENTF_KEYUP;
    if IsExtendedKey(VKCode) then
      ki.dwFlags := ki.dwFlags or KEYEVENTF_EXTENDEDKEY;
  end;

  SendInput(1, Input, SizeOf(TInput));
end;

procedure EmulateKeyPress(VKCode: Byte);
var
  Input: array [0..1] of TInput;
  F: DWord;
begin
  if IsExtendedKey(VKCode) then
    F := KEYEVENTF_EXTENDEDKEY
  else
    F := 0;

  FillChar(Input, SizeOf(Input) {Length(Input)*SizeOf(TInput)}, 0);
  with Input[0] do
  begin
    Itype := INPUT_KEYBOARD;
    ki.wVk := VKCode;
    ki.dwFlags := F;
  end;

  with Input[1] do
  begin
    Itype := INPUT_KEYBOARD;
    ki.wVk := VKCode;
    ki.dwFlags := F or KEYEVENTF_KEYUP;
  end;

  SendInput(2, Input[0], SizeOf(TInput));
end;

procedure EmulateKeyDownSC(ScanCode: Word);
var
  Input: TInput;
begin
  FillChar(Input, SizeOf(TInput), 0);

  with Input do
  begin
    Itype := INPUT_KEYBOARD;
    ki.wScan := ScanCode;
    ki.dwFlags := KEYEVENTF_SCANCODE;
    if (ScanCode shr 8) = $E0 then
      ki.dwFlags := ki.dwFlags or KEYEVENTF_EXTENDEDKEY;
  end;

  SendInput(1, Input, SizeOf(TInput));
end;

procedure EmulateKeyUpSC(ScanCode: Word);
var
  Input: TInput;
begin
  FillChar(Input, SizeOf(TInput), 0);

  with Input do
  begin
    Itype := INPUT_KEYBOARD;
    ki.wScan := ScanCode;
    ki.dwFlags := KEYEVENTF_KEYUP or KEYEVENTF_SCANCODE;
    if (ScanCode shr 8) = $E0 then
      ki.dwFlags := ki.dwFlags or KEYEVENTF_EXTENDEDKEY;
  end;

  SendInput(1, Input, SizeOf(TInput));
end;

procedure EmulateKeyPressSC(ScanCode: Word);
var
  Input: array [0..1] of TInput;
  F: DWord;
begin
  FillChar(Input, SizeOf(Input) {Length(Input)*SizeOf(TInput)}, 0);

  F := KEYEVENTF_SCANCODE;
  if (ScanCode shr 8) = $E0 then
    F := F or KEYEVENTF_EXTENDEDKEY;

  with Input[0] do
  begin
    Itype := INPUT_KEYBOARD;
    ki.wScan := ScanCode;
    ki.dwFlags := F;
  end;

  with Input[1] do
  begin
    Itype := INPUT_KEYBOARD;
    ki.wScan := ScanCode;
    ki.dwFlags := F or KEYEVENTF_KEYUP;
  end;

  SendInput(2, Input[0], SizeOf(TInput));
end;

procedure EmulateKeyPressW(C: WideChar);
var
  Input: array[0..1] of TInput;
begin
  FillChar(Input, SizeOf(Input) {Length(Input)*SizeOf(TInput)}, 0);

  with Input[0] do
  begin
    Itype := INPUT_KEYBOARD;
    ki.wScan := Word(C);
    ki.dwFlags := KEYEVENTF_UNICODE
  end;

  with Input[1] do
  begin
    Itype := INPUT_KEYBOARD;
    ki.wScan := Word(C);
    ki.dwFlags := KEYEVENTF_UNICODE or KEYEVENTF_KEYUP;
  end;

  SendInput(Length(Input), Input[0], SizeOf(TInput));
end;

procedure EmulateString(S: AnsiString);
var
  VK: Byte;
  I, K, L: Integer;
  Input: array of TInput;
  F: DWord;
begin
  L := Length(S) * 2;
  if L = 0 then Exit;

  SetLength(Input, L);
  //     0-
  //FillChar(Input[0], L * SizeOf(TInput), 0);

  I := 0;
  K := 1;
  while I < L do
  begin
    VK := Byte(S[K]);
    Inc(K);

    if IsExtendedKey(VK) then
      F := KEYEVENTF_EXTENDEDKEY
    else
      F := 0;

    with Input[I] do
    begin
      Itype := INPUT_KEYBOARD;
      ki.wVk := VK;
      ki.dwFlags := F;
    end;
    Inc(I);

    with Input[I] do
    begin
      Itype := INPUT_KEYBOARD;
      ki.wVk := VK;
      ki.dwFlags := F or KEYEVENTF_KEYUP
    end;
    Inc(I);
  end;

  SendInput(Length(Input), Input[0], SizeOf(TInput));
end;

procedure EmulateStringW(S: WideString);
var
  SC: Word;
  I, K, L: Integer;
  Input: array of TInput;
begin
  L := Length(S) * 2;
  if L = 0 then Exit;

  SetLength(Input, L);
  //     0-
  //FillChar(Input[0], L * SizeOf(TInput), 0);

  I := 0;
  K := 1;
  while I < L do
  begin
    SC := Word(S[K]);
    Inc(K);

    with Input[I] do
    begin
      Itype := INPUT_KEYBOARD;
      ki.wScan := SC;
      ki.dwFlags := KEYEVENTF_UNICODE;
    end;
    Inc(I);

    with Input[I] do
    begin
      Itype := INPUT_KEYBOARD;
      ki.wScan := SC;
      ki.dwFlags := KEYEVENTF_UNICODE or KEYEVENTF_KEYUP;
    end;
    Inc(I);

  end;

  SendInput(Length(Input), Input[0], SizeOf(TInput));
end;

end.