unit hiKeyEvent;

interface

uses
  Windows, Kol, Share, Debug, SendInp;

type
  THIKeyEvent = class(TDebug)
    private
      function IsKeyPressed(Code: Integer): Integer;
    public
      _prop_Code: Integer;
      _prop_Alt: Integer;
      _prop_Ctrl: Integer;
      _prop_Shift: Integer;
      _prop_Mode: Integer;
      _prop_Text: string;

      _data_Handle: THI_Event;
      _data_Code: THI_Event;
      _data_Alt: THI_Event;
      _data_Ctrl: THI_Event;
      _data_Shift: THI_Event;
      _data_Text: THI_Event;

      procedure _work_doPress(var _Data: TData; Index: Word);
      procedure _work_doPressDown(var _Data: TData; Index: Word);
      procedure _work_doPressUp(var _Data: TData; Index: Word);
      procedure _work_doSendString(var _Data: TData; Index: Word);

      procedure _var_isShift(var _Data: TData; Index: Word);
      procedure _var_isLShift(var _Data: TData; Index: Word);
      procedure _var_isRShift(var _Data: TData; Index: Word);
      procedure _var_isCtrl(var _Data: TData; Index: Word);
      procedure _var_isLCtrl(var _Data: TData; Index: Word);
      procedure _var_isRCtrl(var _Data: TData; Index: Word);
      procedure _var_isAlt(var _Data: TData; Index: Word);
      procedure _var_isLAlt(var _Data: TData; Index: Word);
      procedure _var_isRAlt(var _Data: TData; Index: Word);
      procedure _var_isWinkey(var _Data: TData; Index: Word);
      procedure _var_isLWinkey(var _Data: TData; Index: Word);
      procedure _var_isRWinkey(var _Data: TData; Index: Word);
  end;

implementation

function THIKeyEvent.IsKeyPressed(Code: Integer): Integer;
begin
  Result := 0;
  if GetKeyState(Code) < 0 then Result := 1;
end;

procedure THIKeyEvent._work_doPress;
var
  H, PrevH: HWND;
  Code: Word;
  Shift, Ctrl, Alt: Byte;
begin
  H := ReadInteger(_Data, _data_Handle, 0);
  Code := ReadInteger(_Data, _data_Code, _prop_Code);

  Shift := ReadInteger(_Data, _data_Shift, _prop_Shift);
  Ctrl := ReadInteger(_Data, _data_Ctrl, _prop_Ctrl);
  Alt := ReadInteger(_Data, _data_Alt, _prop_Alt);

  if H <> 0 then
  begin
    PrevH := GetForegroundWindow;
    SetForegroundWindow(H);
  end
  else
    PrevH := 0;

  //TODO:       
  if Shift = 1 then EmulateKeyDown(16); //VK_SHIFT
  if Ctrl = 1 then EmulateKeyDown(17); //VK_CONTROL
  if Alt = 1 then EmulateKeyDown(18); //VK_MENU

  if _prop_Mode = 1 then
    EmulateKeyPressW(WideChar(Code))
  else
    EmulateKeyPress(Code);

  if Alt = 1 then EmulateKeyUp(18);
  if Ctrl = 1 then EmulateKeyUp(17);
  if Shift = 1 then EmulateKeyUp(16);

  if PrevH <> 0 then
    SetForegroundWindow(PrevH);
end;

procedure THIKeyEvent._work_doPressDown;
var
  H, PrevH: HWND;
  Code: Word;
begin
  H := ReadInteger(_Data, _data_Handle, 0);
  Code := ReadInteger(_Data, _data_Code, _prop_Code);

  if H <> 0 then
  begin
    PrevH := GetForegroundWindow;
    SetForegroundWindow(H);
  end
  else
    PrevH := 0;

  EmulateKeyDown(Code);

  if PrevH <> 0 then
    SetForegroundWindow(PrevH);
end;

procedure THIKeyEvent._work_doPressUp;
var
  H, PrevH: HWND;
  Code: Word;
begin
  H := ReadInteger(_Data, _data_Handle, 0);
  Code := ReadInteger(_Data, _data_Code, _prop_Code);

  if H <> 0 then
  begin
    PrevH := GetForegroundWindow;
    SetForegroundWindow(H);
  end
  else
    PrevH := 0;

  EmulateKeyUp(Code);

  if PrevH <> 0 then
    SetForegroundWindow(PrevH);
end;

procedure THIKeyEvent._work_doSendString(var _Data: TData; Index: Word);
var
  H, PrevH: HWND;
  S: string;
begin
  H := ReadInteger(_Data, _data_Handle, 0);
  S := ReadString(_Data, _data_Text, _prop_Text);
  if S = '' then Exit;

  if H <> 0 then
  begin
    PrevH := GetForegroundWindow;
    SetForegroundWindow(H);
  end
  else
    PrevH := 0;

  if _prop_Mode = 1 then
    EmulateStringW(S)
  else
    EmulateString(S);

  if PrevH <> 0 then
    SetForegroundWindow(PrevH);
end;



procedure THIKeyEvent._var_isShift;
begin
  dtInteger(_Data, IsKeyPressed(16));
end;

procedure THIKeyEvent._var_isLShift;
begin
  dtInteger(_Data, IsKeyPressed(160));
end;

procedure THIKeyEvent._var_isRShift;
begin
  dtInteger(_Data, IsKeyPressed(161));
end;

procedure THIKeyEvent._var_isCtrl;
begin
  dtInteger(_Data, IsKeyPressed(17));
end;

procedure THIKeyEvent._var_isLCtrl;
begin
  dtInteger(_Data, IsKeyPressed(162));
end;

procedure THIKeyEvent._var_isRCtrl;
begin
  dtInteger(_Data, IsKeyPressed(163));
end;

procedure THIKeyEvent._var_isAlt;
begin
  dtInteger(_Data, IsKeyPressed(18));
end;

procedure THIKeyEvent._var_isLAlt;
begin
  dtInteger(_Data, IsKeyPressed(164));
end;

procedure THIKeyEvent._var_isRAlt;
begin
  dtInteger(_Data, IsKeyPressed(165));
end;

procedure THIKeyEvent._var_isWinkey;
begin
  dtInteger(_Data, Byte((IsKeyPressed(91) + IsKeyPressed(92)) > 0));
end;

procedure THIKeyEvent._var_isLWinkey;
begin
  dtInteger(_Data, IsKeyPressed(91));
end;

procedure THIKeyEvent._var_isRWinkey;
begin
  dtInteger(_Data, IsKeyPressed(92));
end;

end.
